@@ -329,6 +329,62 @@ def attach(doc):
 	for k,v in ssv.items():
 		doc.user_data[k] = v 
 
+def lempos_type(doc):  # cp from attach, 2022.6.1, to be used in ES indexing 
+	ssv = defaultdict(dict)
+	try:
+		[ ssv[f"rootv-{t.i}"].update ({"type":"rootv", "lempos": f"{t.lemma_}/{t.pos_}", "tag":t.tag_}) for t in doc if t.dep_ =='ROOT']
+		[ ssv[f"vvbg-{t.i}"].update ({"type":"vvbg", "lempos": f"{t.head.lemma_}/{t.head.pos_}", "chunk":f"{t.head.lemma_} {t.text.lower()}", "kp":f"{t.head.lemma_} DOING", "tail":t.lemma})
+			 for t in doc if t.dep_ =='xcomp' and t.head.pos_ == 'VERB' and t.tag_ == 'VBG' and t.lemma_.isalpha() and t.head.lemma_.isalpha()]   #I enjoy smiling. 
+		[ ssv[f"vtov-{t.i}"].update ({"type":"vtov", "lempos": f"{t.head.lemma_}/{t.head.pos_}", "tail":t.text.lower(), "chunk":f"{t.head.lemma_} to {t.text.lower()}", "kp":f"{t.head.lemma_} to DO"}) # begin to DO:go 
+			 for t in doc if t.dep_ =='xcomp' and t.head.pos_ == 'VERB' and t.tag_ == 'VB' and t.i > 1 and t.doc[t.i-1].tag_ == 'TO' and t.head.text.isalpha() and t.head.lemma_.isalpha()]   #I plan to go. 
+		[ ssv[f"mdv-{t.i}"].update ({"type":"mdv","lempos": f"{t.head.lemma_}/{t.head.pos_}", "chunk":f"{t.head.lemma_} {t.lemma_}"}) for t in doc  if t.tag_ == 'MD' and t.dep_ =='aux' and t.head.pos_ == 'VERB' and t.head.lemma_.isalpha()]
+		[ ssv[f"np-{sp.start}"].update ({"type":"np","lempos":sp.root.lemma_.lower()+"/"+sp.root.pos_, "chunk":sp.text.lower()})
+			 for sp in doc.noun_chunks if sp.end - sp.start > 1] # add  #book:np , f"#{sp.root.lemma_.lower()}:np" 
+		[ ssv[f"npone-{sp.start}"].update ({"type":"np","lempos":sp.root.lemma_.lower()+"/"+sp.root.pos_,"chunk":sp.text.lower()}) for sp in doc.noun_chunks  if sp.end - sp.start <= 1]
+		[ ssv[f"vp-{ibeg}"].update ({"type":"vp","lempos":doc[ibeg].lemma_+"/" + doc[ibeg].pos_,"chunk":vp_span(doc,ibeg,iend), "tail":doc[iend-1].lemma_}) for name, ibeg,iend in matchers['vp'](doc)]
+		[ ssv[f"vprt-{ibeg}"].update ({"type":"vprt","lempos":doc[ibeg].lemma_+"/" + doc[ibeg].pos_,"chunk":vp_span(doc,ibeg,iend),})
+			 for name, ibeg,iend in matchers['vprt'](doc)]
+		[ ssv[f"pp-{ibeg}"].update ({"type":"pp","lempos":doc[iend-1].lemma_.lower()+"/" + doc[iend-1].pos_, "chunk":doc[ibeg:iend].text.lower(),})
+			 for name, ibeg,iend in matchers['pp'](doc)]
+		[ ssv[f"ap-{ibeg}"].update ({"type":"ap","lempos":doc[iend-1].lemma_.lower()+"/" + doc[iend-1].pos_,"chunk":doc[ibeg:iend].text.lower(), })
+			for name, ibeg,iend in matchers['ap'](doc)]
+		[ ssv[f"vend-{ibeg}"].update ({"type":"vend","lempos":doc[iend-1].lemma_+"/" + doc[iend-1].pos_,"chunk":vp_span(doc,ibeg,iend), })
+			for name, ibeg,iend in matchers['vend'](doc)]
+		[ ssv[f"vpg-{ibeg}"].update ({"type":"vpg","lempos":doc[ibeg+1].lemma_+"/" + doc[ibeg+1].pos_,"chunk":vp_span(doc,ibeg,iend)})
+			for name, ibeg,iend in matchers['vpg'](doc)]
+		[ ssv[f"svo-{doc[x[0]].i}"].update ({"type":"svo", "chunk":f"{doc[x[1]].lemma_} {doc[x[0]].lemma_} {doc[x[2]].lemma_}", "lempos":doc[x[0]].lemma_ + "/VERB", }) 
+			for name, x in depmatchers['svo'](doc) ]
+		[ ssv[f"sva-{doc[x[0]].i}"].update ({"type":"sva","lempos":doc[x[0]].lemma_ +"/VERB","chunk":f"{doc[x[1]].lemma_} {doc[x[0]].lemma_} {doc[x[2]].lemma_}"})
+			for name, x in depmatchers['sva'](doc) ]
+		[ ssv[f"svx-{doc[x[0]].i}"].update ({"type":"svx","lempos":doc[x[0]].lemma_ +"/VERB","chunk":doc[x[1]].lemma_+' '+doc[x[0]].lemma_+' '+doc[x[2]].lemma_, })
+			for name, x in depmatchers['svx'](doc) ]
+		[ ssv[f"sva-{doc[x[0]].i}"].update ({"type":"sva","lempos":doc[x[0]].lemma_ +"/VERB","chunk":doc[x[1]].lemma_+' '+doc[x[0]].lemma_+' '+doc[x[2]].lemma_, })
+			for name, x in depmatchers['svc'](doc) ]
+		[ ssv[f"sattr-{doc[x[0]].i}"].update ({"type":"sattr","lempos":doc[x[0]].lemma_ +"/VERB","chunk":doc[x[1]].lemma_+' '+doc[x[0]].lemma_+' '+doc[x[2]].lemma_, })
+			for name, x in depmatchers['sattr'](doc) ]
+		[ ssv[f"vpn-{doc[x[0]].i}"].update ({"type":"vpn","lempos":doc[x[0]].lemma_ +"/VERB","chunk":doc[x[0]].lemma_+' '+doc[x[1]].lemma_+' '+doc[x[2]].lemma_, })
+			for name, x in depmatchers['vpn'](doc) ]
+		[ ssv[f"vap-{doc[x[0]].i}"].update ({"type":"vap","lempos":doc[x[1]].lemma_ +"/VERB","chunk":doc[x[0]].lemma_+' '+doc[x[1]].lemma_+' '+doc[x[2]].lemma_, })
+			for name, x in depmatchers['vap'](doc) ]
+		[ ssv[f"vdp-{doc[x[0]].i}"].update ({"type":"vdp","lempos":doc[x[0]].lemma_ +"/VERB","chunk":doc[x[0]].lemma_+' '+doc[x[1]].lemma_+' '+doc[x[2]].lemma_, })
+			for name, x in depmatchers['vdp'](doc) ]
+		[ ssv[f"vnp-{doc[x[0]].i}"].update ({"type":"vnp","lempos":doc[x[0]].lemma_ +"/VERB","chunk":doc[x[0]].lemma_+' '+doc[x[1]].lemma_+' '+doc[x[2]].lemma_, })
+			for name, x in depmatchers['vnp'](doc) ]
+		[ ssv[f"vnpn-{doc[x[0]].i}"].update ({"type":"vnpn","lempos":doc[x[0]].lemma_ +"/VERB","chunk":doc[x[0]].lemma_+' '+doc[x[1]].text.lower()+' '+doc[x[2]].text.lower()+' '+doc[x[3]].text.lower(), })
+			for name, x in depmatchers['vnpn'](doc) ]
+		[ ssv[f"vppn-{doc[x[0]].i}"].update ({"type":"vppn","lempos":doc[x[0]].lemma_ +"/VERB","chunk":doc[x[0]].lemma_+' '+doc[x[1]].text.lower()+' '+doc[x[2]].text.lower()+' '+doc[x[3]].text.lower(), })
+			for name, x in depmatchers['vppn'](doc) ]
+		[ ssv[f"vpnpn-{doc[x[0]].i}"].update ({"type":"vpnpn","lempos":doc[x[0]].lemma_ +"/VERB","chunk":doc[x[0]].lemma_+' '+doc[x[1]].text.lower()+' '+doc[x[2]].text.lower()+' '+doc[x[3]].text.lower()+' '+doc[x[4]].lemma_, })
+			for name, x in depmatchers['vpnpn'](doc) ]
+	except Exception as e:
+		print ( "ex:", e) 
+		exc_type, exc_value, exc_traceback_obj = sys.exc_info()
+		traceback.print_tb(exc_traceback_obj)
+
+	for k,v in ssv.items():
+		doc.user_data[k] = v 
+
+
 hit = lambda ssi, *kvs: [ssi[kv[0]].update({kv[1]: 1}) for kv  in kvs ] 
 def zset_ssi(doc, ssi):  # ssi = defaultdict(Counter)
 	hit(ssi, ("SUM", "snt"))
